@@ -1,42 +1,45 @@
 import numpy as np
 import logging
 import astropy.units as u
+from astropy.table import Table
 from fact.coordinates.utils import horizontal_to_camera as horizontal_to_camera_fact
 
+from .configuration import AICTConfig
+
 
 log = logging.getLogger(__name__)
 
 
 def euclidean_distance(x1, y1, x2, y2):
-    return np.sqrt((x1 - x2)**2 + (y1 - y2)**2)
+    return np.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
 
 
 def convert_to_float32(df):
-    ''' Convert a pandas dataframe to float32, replacing overflows with maxvalues '''
+    """ Convert a pandas dataframe to float32, replacing overflows with maxvalues """
 
-    df = df.astype('float32')
-    df.replace(np.inf, np.finfo('float32').max, inplace=True)
-    df.replace(-np.inf, np.finfo('float32').min, inplace=True)
+    df = df.astype("float32")
+    df.replace(np.inf, np.finfo("float32").max, inplace=True)
+    df.replace(-np.inf, np.finfo("float32").min, inplace=True)
 
     return df
 
 
 def check_valid_rows(df):
-    ''' Check for nans in df, warn if there are any, returns a mask with non-nan rows'''
+    """ Check for nans in df, warn if there are any, returns a mask with non-nan rows"""
     nans = df.isnull()
     valid = ~nans.any(axis=1)
 
     nan_counts = nans.sum()
     if (nan_counts > 0).any():
-        nan_counts_str = ', '.join(f'{k}: {v}' for k, v in nan_counts.items() if v > 0)
-        log.warning('Data contains not-predictable events.')
-        log.warning(f'There are nan-values in columns: {nan_counts_str}')
+        nan_counts_str = ", ".join(f"{k}: {v}" for k, v in nan_counts.items() if v > 0)
+        log.warning("Data contains not-predictable events.")
+        log.warning(f"There are nan-values in columns: {nan_counts_str}")
 
     return valid
 
 
 def calc_true_disp(source_x, source_y, cog_x, cog_y, delta, project_disp=False):
-    '''
+    """
     Calculate the training variables for the disp regressor
 
     Parameters
@@ -58,7 +61,7 @@ def calc_true_disp(source_x, source_y, cog_x, cog_y, delta, project_disp=False):
     -------
     abs_disp: absolute value of disp, either projected or not
     sign_disp: sign of disp
-    '''
+    """
     delta_x = source_x - cog_x
     delta_y = source_y - cog_y
 
@@ -68,18 +71,15 @@ def calc_true_disp(source_x, source_y, cog_x, cog_y, delta, project_disp=False):
     sign_disp = np.sign(true_disp)
 
     if project_disp is False:
-        abs_disp = euclidean_distance(
-            source_x, source_y,
-            cog_x, cog_y
-        )
+        abs_disp = euclidean_distance(source_x, source_y, cog_x, cog_y)
     else:
         abs_disp = np.abs(true_disp)
 
     return abs_disp, sign_disp
 
 
-def convert_units(df, model_config):
-    '''
+def convert_units(df, config):
+    """
     Transforms the columns to the desired units.
 
     Parameters:
@@ -93,90 +93,93 @@ def convert_units(df, model_config):
     Returns:
     --------
     df: The DataFrame with converted values.
-    '''
+    """
     coordinate_units = (
-        (model_config.delta_column, model_config.delta_unit, 'rad'),
-        (model_config.focal_length_column, model_config.focal_length_unit, 'm'),
-        (model_config.source_az_column, model_config.source_az_unit, 'deg'),
-        (model_config.source_zd_column, model_config.source_zd_unit, 'deg'),
-        (model_config.source_alt_column, model_config.source_alt_unit, 'deg'),
-        (model_config.pointing_az_column, model_config.pointing_az_unit, 'deg'),
-        (model_config.pointing_zd_column, model_config.pointing_zd_unit, 'deg'),
-        (model_config.pointing_alt_column, model_config.pointing_alt_unit, 'deg'),
+        (config.delta_column, config.delta_unit, "rad"),
+        (config.focal_length_column, config.focal_length_unit, "m"),
+        (config.source_az_column, config.source_az_unit, "deg"),
+        (config.source_zd_column, config.source_zd_unit, "deg"),
+        (config.source_alt_column, config.source_alt_unit, "deg"),
+        (config.pointing_az_column, config.pointing_az_unit, "deg"),
+        (config.pointing_zd_column, config.pointing_zd_unit, "deg"),
+        (config.pointing_alt_column, config.pointing_alt_unit, "deg"),
     )
     for column, unit, expected_unit in coordinate_units:
-        if column in df.columns and unit != expected_unit:
-            df[column] = u.Quantity(
-                df[column].to_numpy(),
-                unit,
-                copy=False,
-            ).to_value(expected_unit)
-
+        if column in df.columns:
+            if isinstance(df, Table):
+                if df[column].unit:
+                    df[column] = df[column].quantity.to_value(expected_unit)
+            elif unit != expected_unit:
+                df[column] = u.Quantity(
+                    df[column].to_numpy(),
+                    unit,
+                    copy=False,
+                ).to_value(expected_unit)
     return df
 
 
-def get_alt(df, model_config):
-    '''
+def get_alt(df, config: AICTConfig):
+    """
     Return altitude for source and pointing from the df.
     Transforms from zenith distance to altitude if zd is given
-    '''
-    if model_config.source_zd_column:
-        source_alt = 90 - df[model_config.source_zd_column]
+    """
+    if config.source_zd_column:
+        source_alt = 90 - df[config.source_zd_column]
     else:
-        source_alt = df[model_config.source_alt_column]
+        source_alt = df[config.source_alt_column]
 
-    if model_config.pointing_zd_column:
-        pointing_alt = 90 - df[model_config.pointing_zd_column]
+    if config.pointing_zd_column:
+        pointing_alt = 90 - df[config.pointing_zd_column]
     else:
-        pointing_alt = df[model_config.pointing_alt_column]
+        pointing_alt = df[config.pointing_alt_column]
 
     return source_alt, pointing_alt
 
 
-def get_zd(df, model_config):
-    '''
+def get_zd(df, config):
+    """
     Return altitude for source and pointing from the df.
     Transforms from zenith distance to altitude if zd is given
-    '''
-    if model_config.source_alt_column:
-        source_zd = 90 - df[model_config.source_alt_column]
+    """
+    if config.source_alt_column:
+        source_zd = 90 - df[config.source_alt_column]
     else:
-        source_zd = df[model_config.source_zd_column]
+        source_zd = df[config.source_zd_column]
 
-    if model_config.pointing_alt_column:
-        pointing_zd = 90 - df[model_config.pointing_alt_column]
+    if config.pointing_alt_column:
+        pointing_zd = 90 - df[config.pointing_alt_column]
     else:
-        pointing_zd = df[model_config.pointing_zd_column]
+        pointing_zd = df[config.pointing_zd_column]
 
     return source_zd, pointing_zd
 
 
-def horizontal_to_camera(df, model_config):
-    if model_config.coordinate_transformation == 'CTA':
+def horizontal_to_camera(df, config):
+    if config.coordinate_transformation == "CTA":
         from .cta_helpers import horizontal_to_camera_cta_simtel
 
-        alt_source, alt_pointing = get_alt(df, model_config)
+        alt_source, alt_pointing = get_alt(df, config)
         source_x, source_y = horizontal_to_camera_cta_simtel(
-            az=df[model_config.source_az_column],
+            az=df[config.source_az_column],
             alt=alt_source,
-            az_pointing=df[model_config.pointing_az_column],
+            az_pointing=df[config.pointing_az_column],
             alt_pointing=alt_pointing,
-            focal_length=df[model_config.focal_length_column],
+            focal_length=df[config.focal_length_column],
         )
-
-    elif model_config.coordinate_transformation == 'FACT':
-        zd_source, zd_pointing = get_zd(df, model_config)
+    elif config.coordinate_transformation == "FACT":
+        zd_source, zd_pointing = get_zd(df, config)
         source_x, source_y = horizontal_to_camera_fact(
-            az=df[model_config.source_az_column],
+            az=df[config.source_az_column],
             zd=zd_source,
-            az_pointing=df[model_config.pointing_az_column],
+            az_pointing=df[config.pointing_az_column],
             zd_pointing=zd_pointing,
         )
     else:
-        raise ValueError('Unsupported value for coordinate_transformation')
+        raise ValueError("Unsupported value for coordinate_transformation")
 
     return source_x, source_y
 
+
 def delta_error(data_df, model_config):
     df = data_df.copy()
     source_x, source_y = horizontal_to_camera(df, model_config)
@@ -188,7 +191,7 @@ def delta_error(data_df, model_config):
     ####    calculate the difference    ####
     delta_diff = true_delta - df[model_config.delta_column]
     ####    fold pi and -pi into the middle    ####
-    delta_diff[delta_diff < np.pi/2] = delta_diff[delta_diff < np.pi/2] + np.pi
-    delta_diff[delta_diff > np.pi/2] = delta_diff[delta_diff > np.pi/2] - np.pi
+    delta_diff[delta_diff < np.pi / 2] = delta_diff[delta_diff < np.pi / 2] + np.pi
+    delta_diff[delta_diff > np.pi / 2] = delta_diff[delta_diff > np.pi / 2] - np.pi
 
-    return delta_diff
\ No newline at end of file
+    return delta_diff
